#ifndef __CMATH_H
#define __CMATH_H

#include <stdio.h>
#include <stdlib.h>
#include "sp_math.h"

/*  LES NOUVELLES CLASSES POUR SPACE RACER 2000  */
/* We adopt the OpenGL convention: */
/* a0  a4  a8  a12  */
/* a1  a5  a9  a13  */
/* a2  a6 a10  a14  */
/* a3  a7 a11  a15  */

class Sp_Mat4;



/** A 4x4 Matrix compatible with the opengl matrix */
class Sp_Mat4
{
  public:
        /**@name The constructors */
        //@{
        /// the null constructor
    Sp_Mat4();
        /// the opengl constructor
    Sp_Mat4(float *t);
        /// A constrcutor define by a repere
    Sp_Mat4(const Sp_Vecteur &i,const Sp_Vecteur &j,const Sp_Vecteur &k,const Sp_Point &O);
        /// copy constructor
    Sp_Mat4(const Sp_Mat4 &);
        //@}
    
        /// the destructor
    virtual ~Sp_Mat4();
    
    
        /// Set this matrix to Identity
    void Id();
        /// the set function
    void Set(float *);
        /// another set function
    void Set(const Sp_Vecteur &i,const Sp_Vecteur &j,const Sp_Vecteur &k,const Sp_Point &O);
        /// the get function
    void Get(float *);
    
        /// Define the matrix to be a translation
    void Trans(float x,float y,float z);
        /// Set the Matrix to be rotation around axe X
    void SetX(float);
        /// Set the Matrix to be rotation around axe Y
    void SetY(float);
        /// Set the Matrix to be rotation around axe Z
    void SetZ(float);
    
        // pour le traduire en open gl
        //    Sp_Mat4 put() const;
        // la mult

        /**@name The operators */
        //@{
        /// the product with another matrix
    Sp_Mat4 operator*(const Sp_Mat4 &) const;
        /// the product with a vector
    Sp_Vecteur operator*(const Sp_Vecteur &)const;
        /// the product with a point
    Sp_Point operator*(const Sp_Point &) const;
        /// the inverse
    Sp_Mat4 Inverse() const;
        /// the copy
    Sp_Mat4 &operator=(const Sp_Mat4 &);
        //@}
    
        /// Print its informations
    void Print();
        /// its datas
    float *tab;
};


#endif

