#ifndef __CIRCUIT_H
#define __CIRCUIT_H

#include <string>
#include <list>
#include <vector>
#include <stdio.h>
#include <stdlib.h>
#include <GL/glut.h>
#include <iostream.h>
using namespace std;



#include "sp_matrice.h"
#include "sp_math.h"
#include "sp_mathlin.h"
#include "sp_journey.h"



// Nothing Optimised
class Sp_Box;
class Sp_FaceC;
class Sp_CircuitC;



/* What's new:
   - the class Sp_Segment has disappeared
   - A Sp_FaceC is a face composed with 3 points
*/


/* ***************************************
******        A 3D Box          **********
*****************************************/
class Sp_Box
{
  public:
        // the default
    Sp_Box() {};
        // A box wich is a point too
    Sp_Box(const Sp_Point &);
        // the constructor & the destructor
    Sp_Box(float xmi,float xma,float ymi,float yma,float zmi,float zma);
        // the copy constructor
    Sp_Box(const Sp_Box &b);
        // is this point in the box ?
    int IsIn(const Sp_Point &) const;
        // is this box in the box ?
    int IsIn(const Sp_Box &) const;
        // Is there any intersection bewteen this tow box ?
    int IsIntersection(const Sp_Box &) const;
        // Transform  the box to contain the point
    void Add_Point(const Sp_Point &);
        // Transform the box to contain this box
    void Add_Box(const Sp_Box &);
        // Print information about this box
    void Print();
  
  
        // limits of the box
    Sp_Point Pmin;
    Sp_Point Pmax;
};


/* ****************************************
**               an octo Tree            **
******************************************/
// A Sp_Box is divided into precision^3 sub box
template <class T>
class OctoTree
{
        // ******** Public Code *********
public:
        // the functions members
        // the constructors
    OctoTree(Sp_Box &b,int preci);

        // if we use the constructor without args, you must call this function
    OctoTree();   
    void Init(const Sp_Box &b,int preci);
    
        // Insert an element T (Sp_Box is his including volume ) in OctoTree
    void Insert(const T &,const Sp_Box &);
    
        // Essential functions
        // find a list of element T which could contain point P
    void FindList(list<T> &,const Sp_Point &);
        // find a list of element T which could intersect box B
    void FindList(list<T> &,const Sp_Box &);
        // find a list of element T which could intersect Sphere S
    void FindList(list<T> &,const Sp_Sphere &);
    


        // *******  Private code  ********* 
  private:
        // find the coord in the tab  Warning the point must be in the bound box !!!
    void FindCoord(const Sp_Point &,int &x,int &y,int &z) const;    
        // Ajoute une liste a une autre
    void Add_Liste(list<T> &l,list<T> &l2);
    
        // the members datas
  private:
        // the bound box
    Sp_Box box;
        // the hashtable
    list<T> *tab;
        // the precision
    int precision;
};




/* ***************************************
***** a point and the face where it is ***
**************************************** */
class Sp_PointOnFace
{
  public:
    Sp_PointOnFace();
    Sp_PointOnFace(const Sp_Point &p1,Sp_FaceC *f1);
    Sp_Point p;
    Sp_FaceC *face;
};






/* ******************************************
  **  a circuit is a set of  faces **
 ***************************************** */
/** This class stocks informations about the structure of a circuit.
    But it is a Sp_Journey too */
class Sp_CircuitC :public Sp_Journey
{
  public:
        /// Constructor
    Sp_CircuitC();
        /// Add face from the list
    void Add_Face(Sp_FaceC *);

        // make some precomputations
    void PreCompute();
        // Find location 
    int FindLocation(list<Sp_PointOnFace> &l,const Sp_Sphere &,const Sp_Plan &);
    int FindLocationOn(Sp_PointOnFace &P,const Sp_Vecteur &displacement,const Sp_PointOnFace &init);
    
        //the list of faces
    vector<Sp_FaceC *> list_faces;

    
  private:
        // private data

    OctoTree<Sp_FaceC *> octotree_faces;
        // prviate functions
  public:
    
    void FindList(list<Sp_FaceC *> &,const Sp_Point &);
    void FindList(list<Sp_FaceC *> &,const Sp_Box &);
    void FindList(list<Sp_FaceC *> &,const Sp_Sphere &);

};










/* ***************************************
   ** a face is a closed outline **
*************************************** */
/** A face is composed by 3 points and used to build a track \\
    The 3 points are defined as pointers. So when points changed
    you must call the ReCompute() function.
    Version rewritten
 */
class Sp_FaceC
{  
public:

// member functions
    
        /// Constructor
    Sp_FaceC(float *p1,float *p2,float *p3);

        /// This function must be called when a point has changed
    void ReCompute();

        /// Give the normal of this face
    Sp_Vecteur Normale() const {return norm; }
  
        /// Give the center of this face
    Sp_Point Center() const {return center; }

        /// Give the plan of this face
    Sp_Plan Plan() const { return plan; }
    
        /// Is this point in the volume defined by face and diretion ?
    int IsIn(const Sp_Point &,const Sp_Vecteur &) const;
  
        /// Project a point on the face
    Sp_Point Project(const Sp_Point &,const Sp_Vecteur &) const;
  
        /// Project a vecteur on the face
    Sp_Vecteur Project(const Sp_Vecteur &) const;
  
        // Find the point where we quit the face
    int FindIntersection(const Sp_Segment &seg,Sp_Point &P) const;

    
    
        /// Print information for a face
    void Print() const;

        /// Attach this face to a checkpoint
    void AttachToCheckPoint(Sp_CheckPoint *);

        /// This player is on this face
    void Touch(int player,long time);
    
    
// member data
        // Pointers on the 3 points
    float *P1,*P2,*P3;

  
  public:
        // A face is contained in a box
    Sp_Box box; 



private:
        // compute the center
    void Calc_Center();
        // compute the normale
    void Calc_Normale();
        // compute the plan
    void Calc_Plan();
    
        // intern variable
    Sp_Vecteur norm;
    Sp_Point center;
    Sp_Plan  plan;
    Sp_Point Po1;
    Sp_Point Po2;
    Sp_Point Po3;
    Sp_Segment seg[3];

        // Pointer on the checkpoint
    Sp_CheckPoint *checkpoint;

};






#endif
