#include <GL/gl.h>
#include <GL/glu.h>
#include "stk_text.h"

/* ----------------------- Rectangle ----------------------- */
Stk_Rect::Stk_Rect(float xa,float ya,float size_x,float size_y)
{
    x=xa;
    y=ya;
    width=size_x;
    height=size_y;
}




/* ----------------------- GContext ------------------------- */


Stk_GContext::Stk_GContext(float w,float h)
{
    width=w;
    height=h;
}

Stk_GContext::~Stk_GContext()
{}


void Stk_GContext::Lock()
{
        //okiglViewport(1,width,height-1,0);
    
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    gluOrtho2D(0, width,0,height);
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

}

void Stk_GContext::Free()
{
    glDisable(GL_BLEND);
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
}


/* ----------------------- Widget ------------------------- */
Stk_Widget::Stk_Widget(float x,float y, float w,float h)
        :Stk_Rect(x,y,w,h)
{}

Stk_Widget::~Stk_Widget()
{}

void Stk_Widget::Draw()
{}

/* --------------------- Container ------------------------ */
Stk_Container::Stk_Container(float x,float y, float w,float h)
        :Stk_Widget(x,y,w,h)
{}

Stk_Container::~Stk_Container()
{}

void Stk_Container::Add_Widget(Stk_Widget *w)
{
    widgets.push_back(w);
}

void Stk_Container::Rem_Widget(Stk_Widget *w)
{
    widgets.remove(w);
}

void Stk_Container::Draw()
{
    typedef list<Stk_Widget *>::iterator iter;
    
        // we draw all widgets contained
    for(iter i=widgets.begin();i!=widgets.end();i++){
        (*i)->Draw();
    }
}



/* ----------------------- Window ------------------------- */


Stk_Window::Stk_Window(float w,float h)
        :Stk_Container(0,0,w,h)
{
    gc=new Stk_GContext(w,h);
}


Stk_Window::~Stk_Window()
{
    delete gc;
}

void Stk_Window::Draw()
{
    gc->Lock();
    Stk_Container::Draw();
    gc->Free();
}



/* -----------------------  Text  ------------------------- */

Stk_Text::Stk_Text(char *txt,char md,float x,float y,float w,float h)
        :Stk_Widget(x,y,w,h),text(txt)
{
        // we create a default font
    font=new Stk_FontTXF("default.txf");
    allocate_font=1;
        // we compute width of text
    nbchars=text.size();
        // save the mode
    mode=md;
    
}

Stk_Text::Stk_Text(Stk_Font *f,char *txt,char md,float x,float y,float w,float h)
        :Stk_Widget(x,y,w,h),text(txt)
{
        // we create a default font
    font=f;
    allocate_font=0;
        // we compute width of text
    nbchars=text.size();
        // save the mode
    mode=md;
    
}


void Stk_Text::Draw()
{
    glPushMatrix();
    glTranslatef(x,y,0);
    float decalage;
    float sz=font->GetSize();
    unsigned int i=0;
    switch(mode){
        case 0:
                // right mode
            decalage=(width-sz*nbchars);
            while(i<text.size()){
                font->DrawChar(text[i],decalage,0,0);
                decalage+=sz;
                i++;
            }
            break;
        case 1:
                // left mode
            decalage=0;
            while(i<text.size()){
                font->DrawChar(text[i],decalage,0,0);
                decalage+=sz;
                i++;
            }
            break;
        case 2:
                // center mode
            decalage=(width-sz*nbchars)/2;
            while(i<text.size()){
                font->DrawChar(text[i],decalage,0,0);
                decalage+=sz;
                i++;
            }
    }
    glPopMatrix();
}


Stk_Text::~Stk_Text()
{
    if(allocate_font!=0){
        delete font;
    }
    
}


void Stk_Text::SetFontSize(float sz)
{
    font->SetSize(sz);
}

float Stk_Text::GetFontSize()
{
    return font->GetSize();
}
