#ifndef __STK_TEXT
#define __STK_TEXT

#include <list>
#include <string>
using namespace std;

#include "../stk.h"


#define STK_RIGHT  0
#define STK_LEFT   1
#define STK_CENTER 2


/** A Rectangle
 */
class Stk_Rect
{
public:
        // constructor
    Stk_Rect(float xa,float ya,float size_xa,float size_za);
        /// coord x of upper left corner
    float x;
        /// coord y of upper left corner
    float y;
        /// width of rectangle
    float width;
        /// height of rectangle
    float height;
};






/** This is a graphical context
 */
class Stk_GContext
{
public:
        /// Constructor: need width & height
    Stk_GContext(float w, float h);
        /// Destructor
    ~Stk_GContext();
        /// Lock this context
    void Lock();
        /// Free this context
    void Free();
    
private:
        /// Width of the X11 window
    float width;
        /// Height of the X11 window
    float height;
};






/** This is the basis class of all widget object.
 */
class Stk_Widget: public Stk_Rect
{
public:
        /// Constructors
    Stk_Widget(float x,float y,float w,float h);
    
        /// Destructor
    virtual ~Stk_Widget();
    
        /// Draw function
    virtual void Draw();
};





/** This is a container
 */
class Stk_Container: public Stk_Widget
{
public:
        /// Constructor
    Stk_Container(float x,float y,float w,float h);
        /// Destructor
    ~Stk_Container();
    
        /// Draw 
    virtual void Draw();
    
        /// add a widget in list of son widgets
    void Add_Widget(Stk_Widget *);
        /// remove a widget in list of son widgets
    void Rem_Widget(Stk_Widget *);
        /// list of widgets
    list<Stk_Widget *> widgets;
};



/** This is the root of STK widget hiearchy. You must create it before
    doing anything else.
 */
class Stk_Window: public Stk_Container
{
public:
        /// Constructor (width & height)
    Stk_Window(float w,float h);
        /// Destructor
    ~Stk_Window();
        /// Draw
    virtual void Draw();
    
    
private:
        /// the graphical context connected with this window
    Stk_GContext *gc;
    
};

/** This is a widget to draw some string.
 */
class Stk_Text:public Stk_Widget
{
public:
        /// Constructor with default font (very slow)
    Stk_Text(char *txt,char mode,float x,float y,float w,float h);
        /// Constructor 
    Stk_Text(Stk_Font *,char *txt,char mode,float x,float y,float w,float h);
        /// Destructor
    ~Stk_Text();
    
        /// Draw Text
    virtual void Draw();

        /// Set Font Size
    void SetFontSize(float);
        /// Get Font Size
    float GetFontSize();
    
private:
        // the text 
    string  text;
        // the font (&flag to know if we have allocate this font)
    Stk_Font *font;
    char allocate_font;
        /// Mode ( Right:0 Left:1 Center:2 )
    char mode;
        // number of char in string
    float nbchars;
        // h
    
};



#endif
