#ifndef __STK_KEYBOARD
#define __STK_KEYBOARD

#include "../../../config.h"
#include "stk_control.h"
#include "../stk.h"




#define K_UP       100
#define K_DOWN     101
#define K_RIGHT    102
#define K_LEFT     103
#define K_ENTER    105
#define K_QUIT     42


/** a class to read information about the keyboard. 
 It uses XQueryKeymap to read state of keyboard. So a
 X11 window must be opened before to use it.
 This class allows keyboard to send Messages too.
 A message has two fields: \\
 - keys:    number of keys \\
 - pressed: 0 eq no pressed 1 eq pressed \\
 Let explain the protocol: \\
 Key UP       100 \\
 Key DOWN     101 \\
 Key RIGHT    102 \\
 Key LEFT     103 \\
 Key ENTER    105 \\
 */

class Stk_Keyboard: public Stk_Control,public Stk_PassiveSender
{
public:
        /// constructor 
    Stk_Keyboard(Stk_Looper *);
        /// destructor
    virtual ~Stk_Keyboard();
        /// Is XQueryKeymap present on this sytem ?
    int IsPresent() const;
        /// Read the state of keyboard
    void ReadState();
        /// loop: functions wich generates events
    virtual int Loop();
        /// Print information about keyboard
    void Print() const;

        /// Get the position af axe X
    float getPositionAxeX() const;
        /// Get the position of axe Y
    float getPositionAxeY() const;
        /// Get State of button 1
    float getButton1() const;
        /// Get State of button 2
    float getButton2() const;
        /// Get State of button 3
    float getButton3() const;
        /// Get State of button 4
    float getButton4() const;

public: // it must be private
    int key[256];
    
};




#endif
