#include <stdio.h>
#include <stdlib.h>
#include "stk_message.h"


/* ------------------------------------- Message ---------------------------------- */


Stk_Field::Stk_Field(const char *nom)
        :name(nom)
{
    value=0;
}

Stk_Field::~Stk_Field()
{
    if(value!=0){
        delete value;
        value=0;
    }
}



Stk_Message::Stk_Message()
{}

Stk_Message::Stk_Message(int t)
{
    what=t;
}




int Stk_Message::AddInt(const char *name,int anInt)
{
    Stk_Field *field=new Stk_Field(name);
    field->value=new int(anInt);
    fields.push_back(field);
    return 0;
}


int Stk_Message::AddFloat(const char *name,float aFloat)
{
    Stk_Field *field=new Stk_Field(name);
    field->value=new float(aFloat);
    fields.push_back(field);
    return 0;
}


int Stk_Message::FindInt(const char *nom,int *&val)
{
        // we look trough the fields
    typedef list<Stk_Field *>::iterator iter_field;
    for(iter_field i=fields.begin();i!=fields.end();i++){
            // if we found it
        if((*i)->name.compare(nom)==0){
            val=(int *)(*i)->value;
            return 1;
        }
    }
    val=0;
    return 0;
}


int Stk_Message::FindFloat(const char *nom,float *&val)
{
        // we look trough the fields
    typedef list<Stk_Field *>::iterator iter_field;
    for(iter_field i=fields.begin();i!=fields.end();i++){
            // if we found it
        if((*i)->name.compare(nom)==0){
            val=(float *)(*i)->value;
            return 1;
        }
    }
    val=0;
    return 0;
}

Stk_Message::~Stk_Message()
{
        // we look trough the fields
    typedef list<Stk_Field *>::iterator iter_field;
    for(iter_field i=fields.begin();i!=fields.end();i++){
        delete (*i);
    }
}






/* -------------------- template of queue --------------------- */


template <class T>
Stk_Queue<T>::Stk_Queue()
{}

template <class T>
Stk_Queue<T>::~Stk_Queue()
{}


template <class T>
void Stk_Queue<T>::AddElement(T el)
{
    a_elements.push_back(el);
}

template <class T>
void Stk_Queue<T>::RemElement(T el)
{
    r_elements.push_back(el);
}

template <class T>
void Stk_Queue<T>::Sync()
{
        // we remove elements
    while(r_elements.size()!=0){
        r_elements.back()->DetachFromLopper();
        elements.remove(r_elements.back());
        delete r_elements.back();
        r_elements.pop_back();
    }
        // we add elements
    while(a_elements.size()!=0){
        a_elements.back()->AttachToLooper();
        elements.push_back(a_elements.back());
        a_elements.pop_back();
    }
    
}





/* ************************* Handler ************************ */
Stk_Handler::Stk_Handler()
{
    active=0;
}


Stk_Handler::Stk_Handler(Stk_Looper *l)
{
    looper=l;
    active=1;
    looper->AddHandler(this);
}

Stk_Handler::Stk_Handler(Stk_Handler *h)
{
    looper=h->looper;
    active=1;
    looper->AddHandler(this);
}

Stk_Handler::~Stk_Handler()
{
}

void Stk_Handler::Quit()
{
    looper->RemHandler(this);
}

void Stk_Handler::AttachToLooper()
{
}

void Stk_Handler::DetachFromLopper()
{
}

void Stk_Handler::Activate()
{
    active=1;
}

void Stk_Handler::UnActivate()
{
    active=0;
}

int Stk_Handler::IsActive()
{
    return active;
}

void Stk_Handler::MessageReceived(Stk_Message *msg)
{
}


void Stk_Handler::PostMessage(Stk_Message *msg)
{
        // if the handler is active we send it the message
    if(active){
        MessageReceived(msg);
    }
    delete msg;
}



/* *************************** Looper ************************ */
Stk_Looper::Stk_Looper()
        :Stk_Handler()
{
        // don't need to receive message 
    UnActivate();
    looper=this;
    AddHandler(this);
}


void Stk_Looper::Dispatch()
{
    typedef list<Stk_Handler *>::iterator iter_handler;
    while(messages.size()!=0){
            // we look trough handlers
        for(iter_handler i=handlers.elements.begin();i!=handlers.elements.end();i++){
                // if handler active we send it message
            if((*i)->IsActive()){
                (*i)->MessageReceived(messages.front());
            }
        }
            // we remove it
        delete messages.front();
        messages.pop_front();
    }
}

    



void Stk_Looper::PostMessage(Stk_Message *msg)
{
        // add message in the queue
    messages.push_back(msg);
}



int Stk_Looper::Loop()
{
    passivesenders.Sync();
    handlers.Sync();

        // first we generate events for passive event sender
    typedef list<Stk_PassiveSender *>::iterator iter_sender;
    for(iter_sender i=passivesenders.elements.begin();i!=passivesenders.elements.end();i++){
        if((*i)->Loop()==0){
            delete (*i);
            passivesenders.RemElement(*i);            
        }
    }
        // second we dispatch messages received
    Dispatch();


        // then we run the active prog
    if(prog!=0){
        if(prog->Loop()==0){
            prog->Quit();
        }
    }
    return 1;
}





void Stk_Looper::AddHandler(Stk_Handler *handler)
{
    handlers.AddElement(handler);
}


void Stk_Looper::RemHandler(Stk_Handler *handler)
{
    handlers.RemElement(handler);
}


void Stk_Looper::AddPassiveSender(Stk_PassiveSender *ps)
{
    passivesenders.AddElement(ps);
}




/*  **************** The code for the Stk_prog  ************** */


Stk_Prog::Stk_Prog(Stk_Looper *l)
        :Stk_Handler(l)
{
        // My father is God
    father=0;
}



Stk_Prog::Stk_Prog(Stk_Prog *h)
        :Stk_Handler(h)
{
        // My father is only a prog
    father=h;
}

void Stk_Prog::Quit()
{
    Stk_Handler::Quit();
}



void Stk_Prog::AttachToLooper()
{
    if(father!=0)
        father->UnActivate();
    looper->prog=this;
    Activate();
}

void Stk_Prog::DetachFromLopper()
{
    looper->prog=father;
    if(father!=0)
        father->Activate();
}


void Stk_Prog::MessageReceived(Stk_Message *)
{
}



Stk_Prog::~Stk_Prog()
{
}


int Stk_Prog::Loop()
{
    return 1;
}








/* ************************** the code for Stk_PassiveSender *********** */
Stk_PassiveSender::Stk_PassiveSender()
{}


Stk_PassiveSender::Stk_PassiveSender(Stk_Looper *l)
{
    l->AddPassiveSender(this);
    looper=l;
}

int Stk_PassiveSender::Loop()
{
    return 1;
}
void Stk_PassiveSender::PostMessage(Stk_Message *msg)
{
    looper->PostMessage(msg);
}


void Stk_PassiveSender::AttachToLooper() 
{}

void Stk_PassiveSender::DetachFromLopper()
{}



